global simplified_map
simplified_map=".relocate_conf  0x20000000       0x28 load address 0x00003b08"
import struct, binascii, fileinput, subprocess, sys
from os import path

def ask_user_config_to_use(conf) :
    string = "Select ultrasound output data rate :\r\n"
    string += "         ODR (Hz)\r\n"
    rec_nb = 0
    for rec in conf:
        rec_nb += 1
        string += "- %d/       %2s\r\n" % (rec_nb, rec["odr"])
    string += "Enter choice : "

    result = -1

    try:
        result = int(input(string))
        if (result > rec_nb):
            print('Invalid choice, please select a valid configuration')
            result = -1
    except:
        print('Invalid choice, please enter a number')
    return result - 1

def ask_user_sensitivity_to_use():
    string = "What sensitivity do you want to use [0..8] : "

    result = -1

    try:
        result = int(input(string))
        if (result > 8) or (result < 0):
            print('Invalid choice, please select a valid value')
            result = -1
    except:
        print('Invalid choice, please enter a number')
    return result

def ask_user_led_on_to_use():
    string = "LED turn on period in seconds [5..30] : "

    result = -1

    try:
        result = int(input(string))
        if (result > 30) or (result < 5):
            print('Invalid choice, please select a valid value')
            result = -1
    except:
        print('Invalid choice, please enter a number')
    return result

def ask_user_range_to_use():
    string = "Ultrasound range detection in cm. Select integer in range [100..400] : "

    result = -1

    try:
        result = int((int(input(string)) * 60) / 100)
        if (result > 240) or (result < 60):
            print('Invalid choice, please select a valid value')
            result = -1
    except:
        print('Invalid choice, please enter a number')
    return result

def parse_simplified_map(mapfile):
    list = mapfile.split()
    if len(list) == 6:
        return (int(list[5], 16), int(list[2], 16))
    return (0, 0)

def compute_two_complement(in_bytes):
    sum = 0
    for b in in_bytes:
        sum += b
    return ((sum % 256) ^ 0xFF) + 1

def patch_line(cnt, line, parser_version):
    error = 0
    nb_byte,add,typ = struct.unpack(">BHB", binascii.unhexlify(line[1:9]))
    out = struct.pack(">BHB", nb_byte, add, typ)

    if (1 == cnt):
        a,b,c,d,e = struct.unpack("<LLLHH", binascii.unhexlify(line[9:-3]))
        # Check if we know how to parse these data. If we don’t do nothing and exit
        if (a != parser_version):
            error = a
        else:
            b = int((1000 / config_list[user_choice_odr]["odr"]) * 1000)
        out = out + struct.pack("<LLLHH", a, b, c, d, e)
    elif (2 == cnt):
        a,b,c,d,e,f,g,h,i = struct.unpack("<HBBBBBBLL", binascii.unhexlify(line[9:-3]))
        a = config_list[user_choice_odr]["decim"]
        b = user_choice_sensitivity
        c = config_list[user_choice_odr]["bkgd_decim"][0]
        d = config_list[user_choice_odr]["bkgd_decim"][1]
        e = config_list[user_choice_odr]["bkgd_decim"][2]
        # f and g are padding
        h = int((32768 * 1000 / config_list[user_choice_odr]["odr"]) / 1000)
        i = int(1000 / config_list[user_choice_odr]["odr"])
        out += struct.pack("<HBBBBBBLL", a, b, c, d, e, f, g, h, i)
    else:
        a,b,c,d,e,f = struct.unpack("<HHBBBB", binascii.unhexlify(line[9:-3]))
        a = user_choice_range
        b = int(1000 / config_list[user_choice_odr]["odr"])
        c = config_list[user_choice_odr]["odr"]
        d = config_list[user_choice_odr]["odr"] * user_choice_led_on
        e = config_list[user_choice_odr]["odr"] + 1
        out += struct.pack("<HHBBBB", a, b, c, d, e, f)

    out += struct.pack("B", compute_two_complement(out))

    return error, ":" + binascii.hexlify(out).decode("utf-8").upper() + "\n"

if __name__=='__main__':
    parser_version = 0xBEEF0001
    hex_filename = 'module-presence-det.hex'
    flash_filename = 'flash_module_with_jlink.bat'

    # 1. Check all files we need are present. If not prompt user and exit.
    is_file_missing = False
    if (path.exists(hex_filename) == False):
        print("File %s is missing" % hex_filename)
        is_file_missing = True
    if (path.exists(flash_filename) == False):
        print("File %s is missing" % flash_filename)
        is_file_missing = True
    if (is_file_missing):
        sys.exit(-1)

    # 2. Prepare list of available parameters and ask user to make its choice
    add_choice = lambda x,y,z : {"odr":x, "decim":y, "bkgd_decim":z}
    config_list = []
    config_list.append(add_choice(10, 4, (3,2,2)))
    config_list.append(add_choice(5, 4, (3,2,1)))
    config_list.append(add_choice(2, 3, (1,1,1)))
    config_list.append(add_choice(1, 3, (1,1,1)))

    user_choice_odr = -1
    while(user_choice_odr < 0):
        user_choice_odr = ask_user_config_to_use(config_list)

    user_choice_sensitivity = -1
    while(user_choice_sensitivity < 0):
        user_choice_sensitivity = ask_user_sensitivity_to_use()

    user_choice_led_on = -1
    while(user_choice_led_on < 0):
        user_choice_led_on = ask_user_led_on_to_use()

    user_choice_range = -1
    while(user_choice_range < 0):
        user_choice_range = ask_user_range_to_use()

    # 3. Create the new FW with user’s config
    print("Generating new firmware")
    (address, length) = parse_simplified_map(simplified_map)
    error = 0
    for line in fileinput.input(hex_filename, inplace=True):
        if (int(line[3:7], 16) == address):
            error, line = patch_line(1, line, parser_version)
        elif (error == 0) and (int(line[3:7], 16) == address + 0x10):
            error, line = patch_line(2, line, parser_version)
        elif (error == 0) and (int(line[3:7], 16) == address + 0x20):
            error, line = patch_line(3, line, parser_version)
        # Copy line w/o the \n
        print(line[:-1])
    if (error != 0):
        raise NameError("Unsupported FW version 0x%X with parser 0x%X. Please contact support." % (error, parser_version))
    print("   .. Done")

    # 4. Flash new FW if user is willing to
    flash = input("Do you want to flash the new firmware with JLink (y/n) ? ")
    if (flash == "y") or (flash == "yes"):
        process = subprocess.Popen([flash_filename, '-hexpath', hex_filename, '-nopause'],
                     stdout=subprocess.PIPE, 
                     stderr=subprocess.PIPE)
        stdout, stderr = process.communicate()
        print(stderr.decode("utf-8"))
        print("   .. Done")